/*
 *  startsis.s
 *
 *  Start code for the ERC32.
 *
 *  This is based on the file srt0.s provided with the binary
 *  distribution of the SPARC Instruction Simulator (SIS) found
 *  at ftp://ftp.estec.esa.nl/pub/ws/wsd/erc32.
 *
 *  startsis.s,v 1.5 1996/05/28 16:17:35 joel Exp
 */

#include <asm.h>
#include <erc32.h>

/*
 *  Unexpected trap will halt the processor by forcing it to error state
 */

#define BAD_TRAP \
  ta 0; \
  nop; \
  nop; \
  nop;

/*
 *  Software trap. Treat as BAD_TRAP for the time being...
 */

#define SOFT_TRAP BAD_TRAP


/*  .seg    "text" */
  .text
  PUBLIC(start)
  .global start

SYM(start):
start:

/* 
 *  The trap table has to be the first code in a boot PROM.  But because
 *  the Memory Configuration comes up thinking we only have 4K of PROM, we
 *  cannot have a full trap table and still have room left over to 
 *  reprogram the Memory Configuration register correctly.  This file
 *  uses an abbreviated trap which has every entry which might be used
 *  before RTEMS installs its own trap table.
 */


  PUBLIC(trap_table)
SYM(trap_table):

  TRAP( 0, SYM(hard_reset) );                   ! 00 reset trap 
  BAD_TRAP;                                     ! 01 instruction access 
                                                !    exception
  BAD_TRAP;                                     ! 02 illegal instruction
  BAD_TRAP;                                     ! 03 privileged instruction
  BAD_TRAP;                                     ! 04 fp disabled
  TRAP( 5, SYM(window_overflow_trap_handler) ); ! 05 window overflow
  TRAP( 6, SYM(window_underflow_trap_handler) );! 06 window underflow
  BAD_TRAP;                                     ! 07 memory address not aligned
  BAD_TRAP;                                     ! 08 fp exception
  BAD_TRAP;                                     ! 09 data access exception
  BAD_TRAP;                                     ! 0A tag overflow
  BAD_TRAP;                                     ! 0B undefined
  BAD_TRAP;                                     ! 0C undefined
  BAD_TRAP;                                     ! 0D undefined
  BAD_TRAP;                                     ! 0E undefined
  BAD_TRAP;                                     ! 0F undefined
  BAD_TRAP;                                     ! 10 undefined

  /* 
   *  ERC32 defined traps
   */

  BAD_TRAP;                                     ! 11 masked errors
  BAD_TRAP;                                     ! 12 external 1
  BAD_TRAP;                                     ! 13 external 2
  BAD_TRAP;                                     ! 14 UART A RX/TX
  BAD_TRAP;                                     ! 15 UART B RX/TX
  BAD_TRAP;                                     ! 16 correctable memory error
  BAD_TRAP;                                     ! 17 UART error
  BAD_TRAP;                                     ! 18 DMA access error
  BAD_TRAP;                                     ! 19 DMA timeout
  BAD_TRAP;                                     ! 1A external 3
  BAD_TRAP;                                     ! 1B external 4
  BAD_TRAP;                                     ! 1C general purpose timer
  BAD_TRAP;                                     ! 1D real time clock
  BAD_TRAP;                                     ! 1E external 5
  BAD_TRAP;                                     ! 1F watchdog timeout


  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 20 - 23 undefined
  BAD_TRAP;                                     ! 24 cp_disabled
            BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 25 - 27 undefined
  BAD_TRAP;                                     ! 28 cp_exception
            BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 29 - 2B undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 2C - 2F undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 30 - 33 undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 34 - 37 undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 38 - 3B undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 3C - 3F undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 40 - 43 undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 44 - 47 undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 48 - 4B undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 4C - 4F undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 50 - 53 undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 54 - 57 undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 58 - 5B undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 5C - 5F undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 60 - 63 undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 64 - 67 undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 68 - 6B undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 6C - 6F undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 70 - 73 undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 74 - 77 undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 78 - 7B undefined
  BAD_TRAP; BAD_TRAP; BAD_TRAP; BAD_TRAP;       ! 7C - 7F undefined

  /* 
   *  Software traps
   *
   *  NOTE: At the risk of being redundant... this is not a full
   *        table.  The setjmp on the SPARC requires a window flush trap
   *        handler and RTEMS will preserve the entries that were
   *        installed before.
   */

  SOFT_TRAP; SOFT_TRAP; SOFT_TRAP;              ! 80 - 82
  TRAP( 0x83, SYM(window_flush_trap_handler) ); ! 83 flush windows SW trap

   /*
    *  The following entries are not present.  Upon reset, the ERC32 only 
    *  thinks there is 4K of PROM which is EXACTLY the same size a complete
    *  trap table.  Since it was impossible to reprogram the Memory 
    *  Configuration and jump somewhere in 4 instructions, there was no choice
    *  but to eliminate some entries from the trap table used at reset.
    *  Eliminating the following entries freed enough space enough space to 
    *  implement the code required to reprogram the Memory Configuration 
    *  Controller.  
    * 
    *  Not having all the entries is not really problematic since the whole
    *  point of this exercise is to get RTEMS up and running.  As soon as
    *  RTEMS is initialized, it will install its own trap table and this
    *  one is left as baggage anyway.
    */

    /* 
     *  Traps 84 - FF were of the following form.
     *
     *      SOFT_TRAP; SOFT_TRAP; SOFT_TRAP; SOFT_TRAP;   ! 88 - 8B
     */

/*
 *  This is the hard reset code.
 */

#define PSR_INIT   0x10c0       /* Disable traps, set s and ps */
#define TBR_INIT   0
#define WIM_INIT   2
#define STACK_SIZE 16 * 1024

        PUBLIC(hard_reset)
SYM(hard_reset):
        /*
         *  Program the memory configuration register.
         *
         *  The power up default is: 0x00010000
         *
         *    we need to change:
         *      bits 10-12 = for RAM size
         *      bits 18-20 = for ROM size
         *
         *  NOTE:  It would be VERY nice if this information was not
         *         hard-coded here.
         */

        sethi  %hi(0x00010000), %g1
        or     %lo(0x00010000), %g1, %g1 ! g1 = basic Memory Configuration value

        /*
         *  Figure out which is the highest bit set in the RAM_SIZE
         */

        sethi  %hi(_RAM_SIZE), %g2
        or     %lo(_RAM_SIZE), %g2, %g2  ! g2 = size of RAM

        srl    %g2, 18, %g2              ! RAM_SIZE always >= 256K
        mov    0, %g3                    ! g3 = bit number set
ram_size_loop:
        srl    %g2, 1, %g2               ! shift off another address bit
        cmp    %g2, 0                    ! Any bits left?
        bne    ram_size_loop             ! yes, then continue
        add    %g3, 1, %g3               ! increment bit number

        sub    %g3, 1, %g3               ! decrement bit number

        /*
         *  At this point, g3 has the current value for the RAM_SIZE field
         *  in the Memory Configuration Register
         */

        sll    %g3, 10, %g3              ! to the right place for memory config
        or     %g1, %g3, %g1             ! set it in the memory config value
 
        /*
         *  Figure out which is the highest bit set in the PROM_SIZE
         */
 
        sethi  %hi(_PROM_SIZE), %g2
        or     %lo(_PROM_SIZE), %g2, %g2 ! g2 = size of PROM
 
        srl    %g2, 12, %g2              ! PROM_SIZE always >= 4K
        mov    0, %g3                    ! g3 = bit number set
prom_size_loop:
        srl    %g2, 1, %g2               ! shift off another address bit
        cmp    %g2, 0                    ! Any bits left?
        bne    prom_size_loop            ! yes, then continue
        add    %g3, 1, %g3               ! increment bit number
 
        sub    %g3, 1, %g3               ! decrement bit number

        sll    %g3, 18, %g3              ! to the right place for memory config
        or     %g1, %g3, %g1             ! set it in the memory config value

        /*
         *  At this point, g3 has the current value for the PROM_SIZE field
         *  in the Memory Configuration Register
         */

        sethi  %hi(SYM(ERC32_MEC)), %g3  ! g3 = base address of peripherals
        or     %lo(SYM(ERC32_MEC)), %g3, %g3
                                         ! set the Memory Configuration
        st     %g1, [ %g3 + ERC32_MEC_MEMORY_CONFIGURATION_OFFSET ]
        
        set     PSR_INIT, %g1           ! Initialize psr
        mov     %g1, %psr

        set     WIM_INIT, %g1           ! Initialize WIM
        mov     %g1, %wim               

        set     TBR_INIT, %g1           ! Initialize TBR
        mov     %g1, %tbr
        nop
        nop
        nop

        set     PSR_INIT, %g1
        wr      %g1, 0x20, %psr         ! enable traps
        nop
        nop
        nop

        sethi   %hi(stack_space + STACK_SIZE), %g1
        or      %g1,%lo(stack_space + STACK_SIZE),%g1 
                                        ! g1 = top of stack
        mov     %g1, %sp                ! Set stack pointer
        mov     %sp, %fp                ! Set frame pointer
        nop

        /*
         *  Copy the initialized data to RAM
         *
         *  FROM:   _endtext 
         *  TO:     _data_start 
         *  LENGTH: (__bss_start - _data_start) bytes
         */
 

        sethi %hi(_endtext),%g2
        or    %g2,%lo(_endtext),%g2     ! g2 = start of initialized data in ROM

        sethi %hi(_data_start),%g3
        or    %g3,%lo(_data_start),%g3  ! g3 = start of initialized data in RAM

        sethi %hi(__bss_start),%g4
        or    %g4,%lo(__bss_start),%g4  ! g4 = end of initialized data in RAM

copy_data:
        ldd   [ %g2 ], %g6
        std   %g6 , [ %g3 ]             ! copy this double word
        add   %g3, 8, %g3               ! bump the destination pointer
        add   %g2, 8, %g2               ! bump the source pointer
        cmp   %g3, %g4                  ! Is the pointer past the end of dest?
        bl    copy_data
        nop

        /* clear the bss */

        sethi %hi(_edata),%g2
        or    %g2,%lo(_edata),%g2       ! g2 = start of bss
        sethi %hi(_end),%g3
        or    %g3,%lo(_end),%g3         ! g3 = end of bss
        mov   %g0,%g1                   ! so std has two zeros
zerobss:
        std    %g0,[%g2]
        add    %g2,8,%g2
        cmp    %g2,%g3
        bleu,a zerobss
        nop

        mov     %0, %o2                 ! environ
        mov     %0, %o1                 ! argv
        mov     %0, %o0                 ! argc
        call    SYM(main)
        sub     %sp, 0x60, %sp          ! room for main to save args
        nop

        PUBLIC(BSP_fatal_return)
SYM(BSP_fatal_return):
        ta      0                       ! Halt if _main returns ...
        nop

        /*
         *  There does not seem to be a way to get this aligned AND
         *  in the BSS.
         */
 
        .align  32
        .comm   stack_space, STACK_SIZE

/* end of file */
